<?php

namespace Drupal\graphql_core_schema\Plugin\GraphQL\SchemaExtension;

use Drupal\graphql\GraphQL\ResolverBuilder;
use Drupal\graphql\GraphQL\ResolverRegistryInterface;
use Drupal\graphql\Plugin\GraphQL\SchemaExtension\SdlSchemaExtensionPluginBase;
use Drupal\graphql_core_schema\TypeAwareSchemaExtensionInterface;

/**
 * A schema extensions for date formatting.
 *
 * @SchemaExtension(
 *   id = "formatted_date",
 *   name = "Formatted Date and Time",
 *   description = "Provides fields to get PHP and Drupal formatted dates.",
 *   schema = "core_composable"
 * )
 */
class FormattedDateExtension extends SdlSchemaExtensionPluginBase implements TypeAwareSchemaExtensionInterface {

  /**
   * Array of possible types to extend.
   *
   * @var string[]
   */
  const POSSIBLE_TYPES = [
    'FieldItemTypeChanged',
    'FieldItemTypeCreated',
    'FieldItemTypeTimestamp',
    'FieldItemTypeDatetime',
  ];

  /**
   * {@inheritdoc}
   */
  public function getTypeExtensionDefinition(array $types) {
    $extension = [];
    foreach (self::POSSIBLE_TYPES as $type) {
      if (in_array($type, $types)) {
        $extension[] = "extend type $type {
          formatted(format: String, drupalDateFormat: DrupalDateFormat): String!
        }";
      }
    }

    return implode("\n", $extension);
  }

  /**
   * {@inheritdoc}
   */
  public function registerResolvers(ResolverRegistryInterface $registry): void {
    $builder = new ResolverBuilder();

    $resolver = $builder->produce('formatted_date')
      ->map('timestamp', $builder->fromParent())
      ->map('format', $builder->fromArgument('format'))
      ->map('drupalDateFormat', $builder->fromArgument('drupalDateFormat'));

    foreach (self::POSSIBLE_TYPES as $type) {
      $registry->addFieldResolver($type, 'formatted', $resolver);
    }
  }

}
